SystemOrganization addCategory: #'Math-RICompatibility'!

!BoxedFloat64 methodsFor: '*Math-RICompatibility' stamp: 'WernerKassens 3/27/2016 13:14'!
/ aNumber 
	"Primitive. Answer the result of dividing receiver by aNumber.
	Fail if the argument is not a Float. Essential. See Object documentation
	whatIsAPrimitive."

	<primitive: 50>
	aNumber isZero ifTrue: [ ^ ZeroDivide signalWithDividend: self].
	^aNumber adaptToFloat: self andSend: #/! !

!Object methodsFor: '*Math-RICompatibility' stamp: 'WernerKassens 3/27/2016 13:08'!
isIntervalUnion
^false! !

!Object methodsFor: '*Math-RICompatibility' stamp: 'WernerKassens 3/27/2016 13:08'!
isRealInterval
^false! !

!Integer methodsFor: '*Math-RICompatibility' stamp: 'WernerKassens 3/27/2016 13:17'!
// aNumber 
	| q |
	(aNumber isRealInterval or:[aNumber isIntervalUnion ])
	ifTrue: [ ^aNumber adaptToNumber: self andSend: #//] .
	aNumber = 0 ifTrue: [^ (ZeroDivide dividend: self) signal"<- Chg"].
	self = 0 ifTrue: [^ 0].
	q := self quo: aNumber.
	"Refer to the comment in Number|//."
	(q negative
		ifTrue: [q * aNumber ~= self]
		ifFalse: [q = 0 and: [self negative ~= aNumber negative]])
		ifTrue: [^ q - 1"Truncate towards minus infinity."]
		ifFalse: [^ q]! !

!SmallInteger methodsFor: '*Math-RICompatibility' stamp: 'WernerKassens 3/27/2016 13:19'!
quo: aNumber 
	"Primitive. Divide the receiver by the argument and answer with the 
	result. Round the result down towards zero to make it a whole integer. 
	Fail if the argument is 0 or is not a SmallInteger. Optional. See Object 
	documentation whatIsAPrimitive."
	<primitive: 13>
	aNumber isZero ifTrue: [^ (ZeroDivide dividend: self) signal].
	(aNumber isMemberOf: SmallInteger)
		ifFalse: [^ super quo: aNumber].
	(aNumber = -1 and: [self = self class minVal])
		ifTrue: ["result is aLargeInteger" ^ self negated].
	self primitiveFailed! !

!Number methodsFor: '*Math-RICompatibility' stamp: 'WernerKassens 3/27/2016 13:10'!
max: anObject 
^(anObject isKindOf: Magnitude)  ifTrue: [ super max: anObject ] ifFalse: [ anObject adaptToNumber: self andSend: #max: ]! !

!Number methodsFor: '*Math-RICompatibility' stamp: 'WernerKassens 3/27/2016 13:11'!
min: anObject 
^(anObject isKindOf: Magnitude)  ifTrue: [ super min: anObject ] ifFalse: [ anObject adaptToNumber: self andSend: #min: ]! !

!Number methodsFor: '*Math-RICompatibility' stamp: 'WernerKassens 3/27/2016 13:12'!
raisedTo: aNumber 
	"Answer the receiver raised to aNumber."
(aNumber isRealInterval or: [aNumber isIntervalUnion ])
		ifTrue: [ ^aNumber adaptToNumber: self andSend: #raisedTo: ].
	aNumber isInteger ifTrue: [
		"Do the special case of integer power"
		^ self raisedToInteger: aNumber].
	aNumber isFraction ifTrue: [
		"Special case for fraction power"
		^ (self nthRoot: aNumber denominator) raisedToInteger: aNumber numerator ].
	self < 0 ifTrue: [
		^ ArithmeticError signal: 'Negative numbers can''t be raised to float powers.' ].
	0 = aNumber ifTrue: [^ self class one].	"Special case of exponent=0"
	1 = aNumber ifTrue: [^ self].	"Special case of exponent=1"
	0 = self ifTrue: [				"Special case of self = 0"
		aNumber < 0
			ifTrue: [^ (ZeroDivide dividend: self) signal]
			ifFalse: [^ self]].
	^ (aNumber * self ln) exp		"Otherwise use logarithms"! !

!Number methodsFor: '*Math-RICompatibility' stamp: 'WernerKassens 3/27/2016 13:12'!
raisedToInteger: anInteger

	"The 0 raisedToInteger: 0 is an special case. In some contexts must be 1 and in others must
	be handled as an indeterminate form.
	I take the first context because that's the way that was previously handled.
	Maybe further discussion is required on this topic."
	
	|bitProbe result|
	(anInteger isRealInterval or: [anInteger isIntervalUnion ])
		ifTrue: [ ^anInteger adaptToNumber: self andSend: #raisedToInteger: ].
	anInteger negative ifTrue: [^(self raisedToInteger: anInteger negated) reciprocal].
	bitProbe := 1 bitShift: anInteger highBit - 1.
 	result := self class one.
  	[
		(anInteger bitAnd: bitProbe) = 0 ifFalse: [result := result * self].
       bitProbe := bitProbe bitShift: -1.
		bitProbe > 0 ]
	whileTrue: [result := result * result].
	
	^result! !

!SmallFloat64 methodsFor: '*Math-RICompatibility' stamp: 'WernerKassens 3/27/2016 13:15'!
/ aNumber 
	"Primitive. Answer the result of dividing receiver by aNumber.
	Fail if the argument is not a Float. Essential. See Object documentation
	whatIsAPrimitive."

	<primitive: 550>
	aNumber isZero ifTrue: [ ^ ZeroDivide signalWithDividend: self].
	^aNumber adaptToFloat: self andSend: #/! !
